@@ -5,10 +5,12 @@
 
     Base formatter class.
 
-    :copyright: 2006 by Georg Brandl, Armin Ronacher.
-    :license: BSD, see LICENSE for more details.
+    :copyright: Copyright 2006-2012 by the Pygments team, see AUTHORS.
+    :license: BSD, see LICENSE for details.
 """
 
+import codecs
+
 from pygments.util import get_bool_opt
 from pygments.styles import get_style_by_name
 
@@ -44,8 +46,19 @@ class Formatter(object):
         output. If it is "" or None, Unicode strings will be written
         to the output file, which most file-like objects do not
         support (default: None).
+    ``outencoding``
+        Overrides ``encoding`` if given.
     """
 
+    #: Name of the formatter
+    name = None
+
+    #: Shortcuts for the formatter
+    aliases = []
+
+    #: fn match rules
+    filenames = []
+
     #: If True, this formatter outputs Unicode strings when no encoding
     #: option is given.
     unicodeoutput = True
@@ -55,6 +68,7 @@ class Formatter(object):
         self.full  = get_bool_opt(options, 'full', False)
         self.title = options.get('title', '')
         self.encoding = options.get('encoding', None) or None
+        self.encoding = options.get('outencoding', None) or self.encoding
         self.options = options
 
     def get_style_defs(self, arg=''):
@@ -72,4 +86,7 @@ class Formatter(object):
         Format ``tokensource``, an iterable of ``(tokentype, tokenstring)``
         tuples and write it into ``outfile``.
         """
-        raise NotImplementedError()
+        if self.encoding:
+            # wrap the outfile in a StreamWriter
+            outfile = codecs.lookup(self.encoding)[3](outfile)
+        return self.format_unencoded(tokensource, outfile)
